/*
 *
 *	Header of IO operations for quota utilities
 *
 */

#ifndef _QUOTA_IO_H
#define _QUOTA_IO_H

#include <sys/types.h>

#include <linux/quota.h>

/* Latest known versions */
#define INITKNOWNVERSIONS {\
	0,\
	0\
}

/*
 * Definitions for disk quotas imposed on the average user
 * (big brother finally hits Linux).
 *
 * The following constants define the amount of time given a user
 * before the soft limits are treated as hard limits (usually resulting
 * in an allocation failure). The timer is started when the user crosses
 * their soft limit, it is reset when they go below their soft limit.
 */
#define MAX_IQ_TIME  604800	/* (7*24*60*60) 1 week */
#define MAX_DQ_TIME  604800	/* (7*24*60*60) 1 week */

typedef char *dqbuf_t;

struct dquot {
	qid_t dq_id;
	off_t dq_off;

	struct mem_dqblk dq_dqb;
};

extern struct mem_dqinfo info;
extern int io_flags;

#define GETIDINDEX(id, depth) (((id) >> ((DQTREEDEPTH-(depth)-1)*8)) & 0xff)

#define IOFL_QUOTAON	0x01	/* Is quota enabled? */
#define IOFL_INFODIRTY	0x02	/* Did info change? */

#define QIO_ENABLED	(io_flags & IOFL_QUOTAON)
extern inline void mark_quotafile_info_dirty(void)
{
	io_flags |= IOFL_INFODIRTY;
}

/*
 *	Exported functions
 */
/* Get empty buffer */
extern dqbuf_t getdqbuf(void);
/* Free buffer */
extern inline void freedqbuf(dqbuf_t);
/* Check whether given structure represents empty dquot */
extern int empty_dquot(struct disk_dqblk *dq);
/* Initialize all the io */
extern void init_io(char *, char *, int);
/* Initalize io for new file and create it */
extern void new_io(char *, char *, int);
/* Write base information to file */
extern void commit_info(void);
/* Free all io structures, write dirty ones */
extern void free_io(void);
/* Convert disk quota format to memory one */
extern void disk2memdqb(struct mem_dqblk *, struct disk_dqblk *);
/* Convert memory quota format to disk one */
extern void mem2diskdqb(struct disk_dqblk *, struct mem_dqblk *, qid_t);
/* Read block to buffer */
extern void read_blk(uint, dqbuf_t);
/* Write block from buffer */
extern void write_blk(uint, dqbuf_t);
/* Get quota for given user/group (depends on actual file) */
extern void read_dquot(qid_t, struct dquot *);
/* Write dquot to file / delete it when empty / give it to kernel when quota on... */
extern void commit_dquot(struct dquot *, int);
/* Update dqb_itime and dqb_btime if needed */
extern void update_times(struct mem_dqblk *);

#endif	/* _QUOTA_IO_H */

